@@ -1,5 +1,5 @@
 static char RcsId[]         = 
-"@(#)$Header: DevServerMain.c,v 8.14 2000/06/22 14:50:35 goetz Rel $";
+"@(#)$Header: DevServerMain.c,v 8.18 2000/11/12 13:24:03 goetz Rel $";
 
 /*+*******************************************************************
 
@@ -14,12 +14,24 @@ static char RcsId[]         =
  Original   :	March 1991
 
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:35 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:03 $
  $Author: goetz $
 
  $Log:	DevServerMain.c,v $
- * Revision 8.14  2000/06/22  14:50:35  14:50:35  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:03  13:24:03  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:36  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:16  14:50:16  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:51  goetz
