@@ -1,5 +1,5 @@
 static char RcsId[]      =
-"@(#)$Header: svc_api.c,v 8.14 2000/06/22 14:50:30 goetz Rel $";
+"@(#)$Header: svc_api.c,v 8.18 2000/11/12 13:24:03 goetz Rel $";
 /*+*******************************************************************
 
  File:		svc_api.c
@@ -12,13 +12,25 @@ static char RcsId[]      =
 
  Original:	Feb 1994
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:30 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:03 $
 
  $Author: goetz $
 
  $Log:	svc_api.c,v $
- * Revision 8.14  2000/06/22  14:50:30  14:50:30  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:03  13:24:03  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:32  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:16  14:50:16  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:47  goetz
@@ -63,136 +75,6 @@ static char RcsId[]      =
  Revision 8.0  1999/12/28 14:18:31  goetz
  added TANGO support for TACO dev_xxx() calls via -DTANGO for C++
 
- Revision 7.10  1999/11/25 08:27:13  goetz
- replaced fprintf(stderr) with printf; made startup() C++ compatible
-
- Revision 7.9  1999/11/22 20:18:59  goetz
- removed const from gettransient() prototype
-
- Revision 7.8  1999/11/21 20:45:37  goetz
- included all M.Diehl's patches (major changes to gettransient() + main())
-
- Revision 7.7  1999/11/18 22:28:58  goetz
- fixed event multi-client bug, free client handle after event_listen, timeout in dev_synch()
-
- Revision 7.6  1999/11/01 19:05:16  goetz
- make local copy of user name returned by getpwuid() in sec_api.c
-
- Revision 7.5  1999/08/06 17:44:10  goetz
- removed usage of varargs for g++ on solaris and hpux
-
- * Revision 7.4  99/07/09  05:15:27  05:15:27  goetz (Andy Goetz)
- * added M.Diehl's patch to DevServerSig.c to exit() after calling unregister_server()
- * 
- Revision 7.3  1999/06/07 15:26:58  goetz
- fixed bug with multi-nethost reimport, device name stored with nethost
-
- Revision 7.2  1999/05/12 15:21:59  goetz
- changed dev_event_fire() to void; fixed bug in dev_event_unlisten()
-
- * Revision 7.1  99/05/11  15:59:47  15:59:47  goetz (Andy Goetz)
- * replace static declaration of event_client[] array by malloc()
- * 
- Revision 7.0  1999/04/26 07:30:40  goetz
- implemented user events (added event_api.c)
-
- Revision 6.15  1999/03/27 09:00:18  goetz
- asynch_cleanup() only checks for asynch clients; C++ returns output arguments if DS_WARNING
-
- * Revision 6.14  99/02/27  15:21:18  15:21:18  goetz (Andy Goetz)
- * fixed reimport bug; disabled reimport for rpc version 1
- * 
- Revision 6.13  1999/02/05 00:50:25  goetz
- fixed reimport socket+memory leak; ported to aCC on HPUX; suppressed rpc errors
-
- * Revision 6.12  99/01/18  21:39:01  21:39:01  goetz (Andy Goetz)
- * suppressed test for RPC_TIMEDOUT in svc_check()
- * 
- * Revision 6.11  99/01/14  23:14:51  23:14:51  goetz (Andy Goetz)
- * dbase timeout = 4 s; unregister ver. 1; svc_check() handles RPC_TIMEDOUT
- * 
- * Revision 6.10  98/12/22  10:29:44  10:29:44  taurel (E.Taurel)
- * Added support for Solaris and HP C++ native compiler. Remove K_R C support
- * 
- * Revision 6.9  98/12/15  14:05:01  14:05:01  goetz (Andy Goetz)
- * unregister RPC; use pgm no. 1 as flag; dev_pending() checks timeout; extern C prototype
- * 
- * Revision 6.8  98/11/20  14:35:34  14:35:34  goetz (Andy Goetz)
- * unregister API_VERSION (4) correctly now + check return value of svc_register()
- * 
- * Revision 6.7  98/11/20  10:34:59  10:34:59  goetz (Andy Goetz)
- * improved gettrans_ut(); multi-config_flags=0; removde static client_data; fixed linux select() bug; only register pgm # 0,4,5; portmap address == 111
- * 
- * Revision 6.6  98/09/24  09:25:09  09:25:09  goetz (Andy Goetz)
- * fixed stupid type which reintroduced asynchronous memory leak
- * 
- * Revision 6.5  98/08/26  16:20:33  16:20:33  goetz (Andy Goetz)
- * added dynamic NETHOST allocation, fixed bug in db_setup_multi, ported to Irix 6.4
- * 
- * Revision 6.4  98/04/29  10:21:22  10:21:22  goetz (Andy Goetz)
- * asynchronous - memory leak fixed, OS9 bug fixed, stale handle cleanup, pending=300
- * 
- * Revision 6.3  97/12/10  15:28:32  15:28:32  meyer (J.Meyer)
- * Changed select() timeout in rpc_check_host() to 200ms
- * 
- * Revision 6.2  97/12/01  12:59:14  12:59:14  goetz (Andy Goetz)
- * intermediate checkin before opening 5.14.1 branch
- * 
- * Revision 6.1  97/11/20  21:49:24  21:49:24  goetz (Andy Goetz)
- * main() continue even if cannot register asynch rpc
- * 
- * Revision 6.0  97/11/03  17:22:25  17:22:25  goetz (Andy Goetz)
- * merged synchronous and asynchronous branches
- * 
- Revision 5.15  1997/10/24 15:27:02  klotz
- ported to Windows NT 4.0 + NMSDK 5.0 ; tested with TextTalkds ; new DevServerMain.c
-
- * Revision 5.14  97/09/25  17:36:40  17:36:40  goetz (Andy Goetz)
- * fixed
- * 
- Revision 5.13  1997/09/03 17:34:32  meyer
- Added casting for ucc++ in DevServerSig.c
-
- * Revision 5.12  97/09/03  15:54:09  15:54:09  meyer (J.Meyer)
- * Changed OS9 signal handling from intercept() to signal().
- * 
- * Revision 5.11  97/08/05  14:19:24  14:19:24  goetz (Andy Goetz)
- * ported to VxWork 5.2, pgm. no. based on CRC, -m suppressed, -ve errors bug fix
- * 
- * Revision 5.10  97/03/14  17:26:09  17:26:09  dserver ()
- * fixed bug with open file handles after free
- * 
- * Revision 5.8.1.4  97/03/14  17:24:46  17:24:46  goetz (Andy Goetz)
- * fixed bug with open file handles after free
- * 
- * Revision 5.9  97/03/13  11:17:11  11:17:11  klotz (W.D. Klotz)
- * first part of NT4.0 port
- * 
- * Revision 5.8  97/01/31  12:06:20  12:06:20  goetz (Andy Goetz)
- * dev_protocol(), Linux+security bug fixes, dev_import() ignores timeout on NULL procedure, svc_run for OS9
- * 
- * Revision 5.7  97/01/14  08:57:17  08:57:17  goetz (Andy Goetz)
- * added stateless to dev_free(),_raw(),_asyn(),_protocol(),_timeout()
- * 
- * Revision 5.6  97/01/12  18:33:37  18:33:37  goetz (Andy Goetz)
- * dev_import() now stateless i.e. import works even if server is not running
- * 
- * Revision 5.5  97/01/08  17:02:25  17:02:25  goetz (Andy Goetz)
- * Ported to Linux (2.0.9)
- * 
- * Revision 5.4  96/11/27  09:58:12  09:58:12  goetz (Andy Goetz)
- * removed rpc admin interface from DevServerMain.c; putenv() suppressed for _UCC
- * 
- * Revision 5.3  96/11/26  16:24:38  16:24:38  goetz (Andy Goetz)
- * fixed bug (i_nethost=0) in dev_import_local()
- * 
- * Revision 5.2  96/11/26  13:31:00  13:31:00  goetz (Andy Goetz)
- * OICDevice (C++ wrapper for OIC) added + fixed multi-nethost memory leaks
- * 
- * Revision 5.1  96/11/04  11:00:09  11:00:09  goetz (Andy Goetz)
- * added multi nethost support
- * 
-
  Copyright (c) 19901997 by European Synchrotron Radiation Facility, 
                            Grenoble, France
 ********************************************************************-*/
@@ -261,6 +143,11 @@ long 			max_no_of_devices = 0;
  */
 static long	asyn_error;
 
+/*
+ * Dynamic error stack
+ */
+extern char *dev_error_stack;
+
 /*
  * Internal Functions
  */
@@ -486,6 +373,7 @@ _dev_free_out * _DLLFunc rpc_dev_free_4 (_dev_free_in *dev_free_in)
 _client_data * _DLLFunc rpc_dev_putget_4 (_server_data *server_data)
 {
 	static _client_data	client_data;
+	static DevVarArgument	vararg[1];
 #ifndef __cplusplus
 /*
  * OIC version
@@ -511,17 +399,23 @@ _client_data * _DLLFunc rpc_dev_putget_4 (_server_data *server_data)
 	    server_data->ds_id);
 #endif /* EBUG */
 
-	/*
-    * Free and preset the structure for outgoing arguments.
-    */
+/*
+ * Free and preset the structure for outgoing arguments.
+ */
 
 	free (client_data.argout);
 	memset ((char *)&client_data,0,sizeof(client_data));
+/*
+ * Clear dynamic error stack
+ */
+        if (dev_error_stack != NULL) free(dev_error_stack);
+	dev_error_stack = NULL;
 
 
-	/*
-    * Split up the device identification.
-    */
+
+/*
+ * Split up the device identification.
+ */
 
 	if (read_device_id (server_data->ds_id, &ds_id, &connection_id, 
 	    &client_data.error) == DS_NOTOK)
@@ -631,7 +525,7 @@ _client_data * _DLLFunc rpc_dev_putget_4 (_server_data *server_data)
                                              &client_data.error);
 #endif /* __cplusplus */
 
-	/*
+/*
  *  Because in case of error the status of the outgoing arguments
  *  is undefined, initialise argout to NULL before serialising.
  */
@@ -641,6 +535,17 @@ _client_data * _DLLFunc rpc_dev_putget_4 (_server_data *server_data)
 		free (client_data.argout);
 		client_data.argout = NULL;
 	}
+/*
+ * if a dynamic error has been generated tag it onto the end of
+ * the client_data structure returned
+ */
+	if (dev_error_stack != NULL)
+	{
+		vararg[0].argument_type = D_STRING_TYPE;
+		vararg[0].argument = (DevArgument)&dev_error_stack;
+		client_data.var_argument.length = 1;
+		client_data.var_argument.sequence = vararg;
+	}
 
 	return (&client_data);
 }
@@ -668,6 +573,7 @@ _client_data * _DLLFunc rpc_dev_putget_4 (_server_data *server_data)
 _client_data * _DLLFunc rpc_dev_put_4 (_server_data *server_data)
 {
 	static _client_data	client_data;
+	static DevVarArgument	vararg[1];
 #ifndef __cplusplus
 /*
  * OIC version
@@ -690,15 +596,21 @@ _client_data * _DLLFunc rpc_dev_put_4 (_server_data *server_data)
 	    server_data->ds_id);
 #endif /* EBUG */
 
-	/*
+/*
  *  initialise outgoing arguments 
  */
 
 	memset ((char *)&client_data,0,sizeof(client_data));
 	client_data.argout_type = D_VOID_TYPE;
 	client_data.argout = NULL;
+/*
+ * Clear dynamic error stack
+ */
+        if (dev_error_stack != NULL) free(dev_error_stack);
+	dev_error_stack = NULL;
 
-	/*
+
+   /*
     * Split up the device identification.
     */
 
@@ -765,6 +677,18 @@ _client_data * _DLLFunc rpc_dev_put_4 (_server_data *server_data)
 	                                     client_data.argout_type,
 	                                     &client_data.error);
 #endif /* __cplusplus */
+/*
+ * if a dynamic error has been generated tag it onto the end of
+ * the client_data structure returned
+ */
+	if (dev_error_stack != NULL)
+	{
+		vararg[0].argument_type = D_STRING_TYPE;
+		vararg[0].argument = (DevArgument)&dev_error_stack;
+		client_data.var_argument.length = 1;
+		client_data.var_argument.sequence = vararg;
+	}
+
 
 	return (&client_data);
 }
