@@ -13,13 +13,25 @@
  Original:	June 1992
 
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:27 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:02 $
 
  $Author: goetz $
 
  $Log:	ApiP.h,v $
- * Revision 8.14  2000/06/22  14:50:27  14:50:27  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:02  13:24:02  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:28  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:15  14:50:15  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:44  goetz
@@ -160,7 +172,7 @@
  */
 #ifdef _IDENT
 static char ApiPh[] =
-"@(#)$Header: ApiP.h,v 8.14 2000/06/22 14:50:27 goetz Rel $";
+"@(#)$Header: ApiP.h,v 8.18 2000/11/12 13:24:02 goetz Rel $";
 #endif /* _IDENT */
 
 
@@ -559,6 +571,14 @@ long tango_dev_putget_raw(devserver,long,void*,long,void*,long,long*);
 long tango_dev_cmd_query(devserver,DevVarCmdArray*,long*); 
 long tango_dev_ping(devserver,long*); 
 
+/* used by TANGO */
+
+long dev_notimported_init(char*, long, long, devserver*, long*);
+
+/* dynamic error string  */
+
+long dev_error_push PT_( (char *error_string) );
+
 #ifdef __cplusplus
 }
 #endif
