@@ -1,4 +1,4 @@
-static char RcsId[] = "@(#)$Header: asyn_api.c,v 8.14 2000/06/22 14:50:21 goetz Rel $";
+static char RcsId[] = "@(#)$Header: asyn_api.c,v 8.18 2000/11/12 13:24:01 goetz Rel $";
 /*+*******************************************************************
 
  File       :	asyn_api.c
@@ -25,13 +25,25 @@ static char RcsId[] = "@(#)$Header: asyn_api.c,v 8.14 2000/06/22 14:50:21 goetz
 
  Original   :	January 1997
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:21 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:01 $
 
  $Author: goetz $
 
  $Log:	asyn_api.c,v $
- * Revision 8.14  2000/06/22  14:50:21  14:50:21  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:01  13:24:01  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:24  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:12  14:50:12  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:40  goetz
@@ -76,90 +88,8 @@ static char RcsId[] = "@(#)$Header: asyn_api.c,v 8.14 2000/06/22 14:50:21 goetz
  Revision 8.0  1999/12/28 14:18:21  goetz
  added TANGO support for TACO dev_xxx() calls via -DTANGO for C++
 
- Revision 7.10  1999/11/25 08:27:09  goetz
- replaced fprintf(stderr) with printf; made startup() C++ compatible
-
- Revision 7.9  1999/11/22 20:18:55  goetz
- removed const from gettransient() prototype
-
- Revision 7.8  1999/11/21 20:45:34  goetz
- included all M.Diehl's patches (major changes to gettransient() + main())
-
- Revision 7.7  1999/11/18 22:28:53  goetz
- fixed event multi-client bug, free client handle after event_listen, timeout in dev_synch()
-
- Revision 7.6  1999/11/01 19:05:14  goetz
- make local copy of user name returned by getpwuid() in sec_api.c
-
- Revision 7.5  1999/08/06 17:44:03  goetz
- removed usage of varargs for g++ on solaris and hpux
-
- * Revision 7.4  99/07/09  05:15:25  05:15:25  goetz (Andy Goetz)
- * added M.Diehl's patch to DevServerSig.c to exit() after calling unregister_server()
- * 
- Revision 7.3  1999/06/07 15:26:52  goetz
- fixed bug with multi-nethost reimport, device name stored with nethost
-
- Revision 7.2  1999/05/12 15:21:51  goetz
- changed dev_event_fire() to void; fixed bug in dev_event_unlisten()
-
- * Revision 7.1  99/05/11  15:59:39  15:59:39  goetz (Andy Goetz)
- * replace static declaration of event_client[] array by malloc()
- * 
- Revision 7.0  1999/04/26 07:30:33  goetz
- implemented user events (added event_api.c)
-
- Revision 6.15  1999/03/27 09:00:13  goetz
- asynch_cleanup() only checks for asynch clients; C++ returns output arguments if DS_WARNING
-
- * Revision 6.14  99/02/27  15:21:12  15:21:12  goetz (Andy Goetz)
- * fixed reimport bug; disabled reimport for rpc version 1
- * 
- Revision 6.13  1999/02/05 00:50:20  goetz
- fixed reimport socket+memory leak; ported to aCC on HPUX; suppressed rpc errors
-
- * Revision 6.12  99/01/18  21:38:55  21:38:55  goetz (Andy Goetz)
- * suppressed test for RPC_TIMEDOUT in svc_check()
- * 
- * Revision 6.11  99/01/14  23:14:45  23:14:45  goetz (Andy Goetz)
- * dbase timeout = 4 s; unregister ver. 1; svc_check() handles RPC_TIMEDOUT
- * 
- * Revision 6.10  98/12/22  10:29:39  10:29:39  taurel (E.Taurel)
- * Added support for Solaris and HP C++ native compiler. Remove K_R C support
- * 
- * Revision 6.9  98/12/15  14:04:54  14:04:54  goetz (Andy Goetz)
- * unregister RPC; use pgm no. 1 as flag; dev_pending() checks timeout; extern C prototype
- * 
- * Revision 6.8  98/11/20  14:35:29  14:35:29  goetz (Andy Goetz)
- * unregister API_VERSION (4) correctly now + check return value of svc_register()
- * 
- * Revision 6.7  98/11/20  10:34:54  10:34:54  goetz (Andy Goetz)
- * improved gettrans_ut(); multi-config_flags=0; removde static client_data; fixed linux select() bug; only register pgm # 0,4,5; portmap address == 111
- * 
- * Revision 6.6  98/09/24  09:25:05  09:25:05  goetz (Andy Goetz)
- * fixed stupid type which reintroduced asynchronous memory leak
- * 
- * Revision 6.5  98/08/26  16:20:30  16:20:30  goetz (Andy Goetz)
- * added dynamic NETHOST allocation, fixed bug in db_setup_multi, ported to Irix 6.4
- * 
- * Revision 6.4  98/04/29  10:21:18  10:21:18  goetz (Andy Goetz)
- * asynchronous - memory leak fixed, OS9 bug fixed, stale handle cleanup, pending=300
- * 
- * Revision 6.3  97/12/10  15:28:28  15:28:28  meyer (J.Meyer)
- * Changed select() timeout in rpc_check_host() to 200ms
- * 
- * Revision 6.2  97/12/01  12:59:09  12:59:09  goetz (Andy Goetz)
- * intermediate checkin before opening 5.14.1 branch
- * 
- * Revision 6.1  97/11/20  21:49:19  21:49:19  goetz (Andy Goetz)
- * main() continue even if cannot register asynch rpc
- * 
- * Revision 6.0  97/11/03  17:22:21  17:22:21  goetz (Andy Goetz)
- * merged synchronous and asynchronous branches
- * 
-
- Copyright (c) 1997 by European Synchrotron Radiation Facility,
-                       Grenoble, France
+ Copyright (c) 1997-2000 by European Synchrotron Radiation Facility,
+                            Grenoble, France
 
 ********************************************************************-*/
 
@@ -244,6 +174,12 @@ extern long synch_svc_udp_sock;
 extern long synch_svc_tcp_sock;
 long asynch_svc_tcp_sock;
 
+/* 
+ * dynamic error variables
+ */
+extern char *dev_error_stack;
+extern char *dev_error_string;
+
 /*+**********************************************************************
  Function   :	extern long dev_putget_asyn()
 
@@ -313,9 +249,17 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
                 return(DS_NOTOK);
         }
 #endif /* TANGO */
-	/*
-	 *  check data types
-	 */
+/*
+ * make sure dynamic error is initialised to nothing
+ */
+	if (dev_error_string != NULL)
+	{
+		free(dev_error_string);
+		dev_error_string = NULL;
+	}
+/*
+ *  check data types
+ */
 
 	if (  argin_type < 0 || argout_type < 0)
 	{
@@ -337,17 +281,17 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 	nethost = &multi_nethost[i_nethost];
 
 
-	/*
-	 * Identify a local device.
-	 * Device is local if local_flag == True.
-	 */
+/*
+ * Identify a local device.
+ * Device is local if local_flag == True.
+ */
 
 	local_flag =(_Int)( ds->ds_id >> LOCALACCESS_SHIFT);
 	local_flag = local_flag & LOCALACCESS_MASK;
 
-	/*
-	 * Verify the RPC connection if the device is not local.
-	 */
+/*
+ * Verify the RPC connection if the device is not local.
+ */
 
 	if ( local_flag != True )
 	{
@@ -356,10 +300,10 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 			return (DS_NOTOK);
 		}
 	}
-	/*
-	 * If the security system is configured, 
-	 * verify the security key
-	 */
+/*
+ * If the security system is configured, 
+ * verify the security key
+ */
 
 	if ( nethost->config_flags.security == True )
 	{
@@ -380,10 +324,10 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 		if (rstatus != DS_OK) return(DS_NOTOK);
 	}
 	
-	/*
-	 *  make sure that the asynchronous version of the device
-	 *  server is imported.
-	 */
+/*
+ *  make sure that the asynchronous version of the device
+ *  server is imported.
+ */
 
 	if (asynch_server_import(ds,error) != DS_OK)
 	{
@@ -401,10 +345,10 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 	}
 
 
-	/*
-	 *  fill in data transfer structures server_data
-	 *  and client_data.
-	 */
+/*
+ *  fill in data transfer structures server_data
+ *  and client_data.
+ */
 
 	server_data.ds_id	= ds->ds_id;
 	server_data.client_id	= client_id;
@@ -470,10 +414,10 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
  *
  */
 
-	/*
-	 * Check if the device is a local or a remote device.
-	 * If local, call the local putget function.
-	 */
+/*
+ * Check if the device is a local or a remote device.
+ * If local, call the local putget function.
+ */
 
 	if ( local_flag == True )
 	{
@@ -490,11 +434,11 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 		return(DS_NOTOK);
 	}
 
-	/*
-	 *  call RPC_DEV_PUTGET_ASYN using the client handle which has
-	 *  been opened to the asynchronous service of the server and
-	 *  ONE-WAY rpc i.e. xdr routine = xdr_void && timeout = zero
-	 */
+/*
+ *  call RPC_DEV_PUTGET_ASYN using the client handle which has
+ *  been opened to the asynchronous service of the server and
+ *  ONE-WAY rpc i.e. xdr routine = xdr_void && timeout = zero
+ */
 
 	clnt_stat = clnt_call (ds->asynch_clnt, RPC_DEV_PUTGET_ASYN,
 		    (xdrproc_t)xdr__server_data, (caddr_t) &server_data,
@@ -504,9 +448,9 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
                         "\ndev_putget_asyn() : clnt_stat %d\n",clnt_stat);
 #endif /* EBUG */
 
-	/*
-	 * Check for errors on the RPC connection.
-	 */
+/*
+ * Check for errors on the RPC connection.
+ */
 
 	if ( clnt_stat != RPC_TIMEDOUT)
 	if ( dev_rpc_error (ds, clnt_stat, error) == DS_NOTOK )
@@ -520,9 +464,9 @@ long _DLLFunc dev_putget_asyn (devserver ds, long cmd,
 		return (DS_NOTOK);
 	}
 
-	/*
-	 * return the asynch_id to the client for identification purposes
-	 */
+/*
+ * return the asynch_id to the client for identification purposes
+ */
 
 	*asynch_id_ptr = asynch_id;
 
@@ -600,9 +544,9 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
                 return(DS_NOTOK);
         } 
 #endif /* TANGO */
-	/*
-	 *  check data types
-	 */
+/*
+ *  check data types
+ */
 
 	if (  argin_type < 0 || argout_type < 0)
 	{
@@ -624,17 +568,17 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 	nethost = &multi_nethost[i_nethost];
 
 
-	/*
-	 * Identify a local device.
-	 * Device is local if local_flag == True.
-	 */
+/*
+ * Identify a local device.
+ * Device is local if local_flag == True.
+ */
 
 	local_flag =(_Int)( ds->ds_id >> LOCALACCESS_SHIFT);
 	local_flag = local_flag & LOCALACCESS_MASK;
 
-	/*
-	 * Verify the RPC connection if the device is not local.
-	 */
+/*
+ * Verify the RPC connection if the device is not local.
+ */
 
 	if ( local_flag != True )
 	{
@@ -643,10 +587,10 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 			return (DS_NOTOK);
 		}
 	}
-	/*
-	 * If the security system is configured, 
-	 * verify the security key
-	 */
+/*
+ * If the security system is configured, 
+ * verify the security key
+ */
 
 	if ( nethost->config_flags.security == True )
 	{
@@ -667,10 +611,10 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 		if (rstatus != DS_OK) return(DS_NOTOK);
 	}
 	
-	/*
-	 *  make sure that the asynchronous version of the device
-	 *  server is imported.
-	 */
+/*
+ *  make sure that the asynchronous version of the device
+ *  server is imported.
+ */
 
 	if (asynch_server_import(ds,error) != DS_OK)
 	{
@@ -688,10 +632,10 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 	}
 
 
-	/*
-	 *  fill in data transfer structures server_data
-	 *  and client_data.
-	 */
+/*
+ *  fill in data transfer structures server_data
+ *  and client_data.
+ */
 
 	server_data.ds_id	= ds->ds_id;
 	server_data.client_id	= client_id;
@@ -757,10 +701,10 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
  *
  */
 
-	/*
-	 * Check if the device is a local or a remote device.
-	 * If local, call the local putget function.
-	 */
+/*
+ * Check if the device is a local or a remote device.
+ * If local, call the local putget function.
+ */
 
 	if ( local_flag == True )
 	{
@@ -777,11 +721,11 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 		return(DS_NOTOK);
 	}
 
-	/*
-	 *  call RPC_DEV_PUTGET_RAW_ASYN using the client handle which has
-	 *  been opened to the asynchronous service of the server and
-	 *  ONE-WAY rpc i.e. xdr routine = xdr_void && timeout = zero
-	 */
+/*
+ *  call RPC_DEV_PUTGET_RAW_ASYN using the client handle which has
+ *  been opened to the asynchronous service of the server and
+ *  ONE-WAY rpc i.e. xdr routine = xdr_void && timeout = zero
+ */
 
 	clnt_stat = clnt_call (ds->asynch_clnt, RPC_DEV_PUTGET_RAW_ASYN,
 		    (xdrproc_t)xdr__server_data, (caddr_t) &server_data,
@@ -802,9 +746,9 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
                         "\ndev_putget_asyn() : clnt_stat %d\n",clnt_stat);
 #endif /* EBUG */
 
-	/*
-	 * Check for errors on the RPC connection.
-	 */
+/*
+ * Check for errors on the RPC connection.
+ */
 
 	if ( dev_rpc_error (ds, clnt_stat, error) == DS_NOTOK )
 	{
@@ -817,9 +761,9 @@ long _DLLFunc dev_putget_raw_asyn (devserver ds, long cmd,
 		return (DS_NOTOK);
 	}
 
-	/*
-	 * return the asynch_id to the client for identification purposes
-	 */
+/*
+ * return the asynch_id to the client for identification purposes
+ */
 
 	*asynch_id_ptr = asynch_id;
 
@@ -885,9 +829,9 @@ static void _WINAPI devserver_prog_5 (struct svc_req *rqstp, SVCXPRT *transp)
             "\ndevserver_prog_5() : rqstp->rq_proc=%d\n",rqstp->rq_proc);
 #endif /* EBUG */
 
-	/*
-	 *  call the right server routine
-	 */
+/*
+ *  call the right server routine
+ */
 
 	switch (rqstp->rq_proc) {
 	case NULLPROC:
@@ -1188,6 +1132,15 @@ static void _WINAPI devserver_prog_5 (struct svc_req *rqstp, SVCXPRT *transp)
 			   asynch_client_data.error = client_data->error;
 			   asynch_client_data.argout_type = client_data->argout_type;
 			   asynch_client_data.argout = client_data->argout;
+/*
+ * if dynamic error then tag it onto the end of the client_data
+ */
+			   if (dev_error_stack != NULL)
+			   {
+			   	vararg[iarg].argument_type      = D_STRING_TYPE;
+				vararg[iarg].argument = (DevArgument)&dev_error_stack;
+				iarg++;
+			   }
 			   asynch_client_data.var_argument.length = iarg; 
 			   asynch_client_data.var_argument.sequence = vararg;
 		
@@ -2157,6 +2110,15 @@ _asynch_client_data* _DLLFunc rpc_asynch_reply_5(_asynch_client_data *asynch_cli
 		cb_data.time.tv_sec = *(long*)(asynch_client_data->var_argument.sequence[iarg].argument);
 		iarg++;
 		cb_data.time.tv_usec = *(long*)(asynch_client_data->var_argument.sequence[iarg].argument);
+		iarg++;
+/*
+ * unpack dynamic error message if present 
+ */
+		if (asynch_client_data->var_argument.length == 3)
+		{
+			dev_error_string = (char*)malloc(strlen(*(char**)asynch_client_data->var_argument.sequence[iarg].argument)+1);
+			sprintf(dev_error_string,*(char**)asynch_client_data->var_argument.sequence[iarg].argument);
+		}
 
 		(*client_asynch_request.args[asynch_index].callback) 
 		 (client_asynch_request.args[asynch_index].ds, 
