@@ -1,4 +1,4 @@
-static char RcsId[] = "@(#)$Header: event_api.c,v 8.14 2000/06/22 14:50:23 goetz Rel $";
+static char RcsId[] = "@(#)$Header: event_api.c,v 8.18 2000/11/12 13:24:01 goetz Rel $";
 /*+*******************************************************************
 
  File       :	event_api.c
@@ -33,13 +33,25 @@ static char RcsId[] = "@(#)$Header: event_api.c,v 8.14 2000/06/22 14:50:23 goetz
 
  Original   :	April 1999
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:23 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:01 $
 
  $Author: goetz $
 
  $Log:	event_api.c,v $
- * Revision 8.14  2000/06/22  14:50:23  14:50:23  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:01  13:24:01  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:26  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:12  14:50:12  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:41  goetz
