@@ -1,5 +1,5 @@
 static char RcsId[] =
-"@(#)$Header: sec_api.c,v 8.14 2000/06/22 14:50:19 goetz Rel $";
+"@(#)$Header: sec_api.c,v 8.18 2000/11/12 13:24:01 goetz Rel $";
 /*+*******************************************************************
 
  File       :   sec_api.c
@@ -12,13 +12,25 @@ static char RcsId[] =
 
  Original   :   December 1993
 
- $Revision: 8.14 $
- $Date: 2000/06/22 14:50:19 $
+ $Revision: 8.18 $
+ $Date: 2000/11/12 13:24:01 $
 
  $Author: goetz $
 
  $Log:	sec_api.c,v $
- * Revision 8.14  2000/06/22  14:50:19  14:50:19  goetz (Andrew GOETZ)
+ * Revision 8.18  2000/11/12  13:24:01  13:24:01  goetz (Andy Goetz)
+ * added dynamic error handling; improved TANGO error treatment
+ * 
+ Revision 8.17  1900/09/26 13:36:23  goetz
+ tango_dev_putget() and tango_dev_putget_raw() implement immediate reconnection
+
+ * Revision 8.16  2000/09/25  14:50:12  14:50:12  goetz (Andy Goetz)
+ * tango_api implements TANGO_HOST, stateless import and reimport
+ * 
+ * Revision 8.15  20/0./7.  1.:6.:9.  1.:6.:9.  goetz (Andy Goetz)
+ * changed from tango database to host mysql, added support for float and double arrays
+ * 
+ * Revision 8.14  20/0./6.  2.:4.:0.  2.:4.:0.  goetz (Andy Goetz)
  * fixed bug in dev_event_listen() which caused server to crash
  * 
  Revision 8.13  2000/06/02 21:56:38  goetz
